
/*

     Read a fixed size lump of data from standard in, and put it in 
     the argument file.  Result is 1 if an error occurs, 2 if standard
     in closes before lump size was read.

*/

#include <stdio.h>
#include <fcntl.h>

#define BUFFER_SIZE 102400

static char *usage        = "USAGE: savelump <file>\n" ;

extern int  errno, sys_nerr ;
extern char *sys_errlist [] ;

void sys_assert (condition, usr_message)
/*

  Test `condition'.  Abort program, reporting `usr_message', if it fails.

*/
int condition ;
char *usr_message ;
{
  if (!condition)
  {
    fprintf (stderr, "ERROR: %s", usr_message) ;
    if ((errno > 0) && (errno < sys_nerr))
      fprintf (stderr, "... %s", sys_errlist [errno]) ;
    fprintf (stderr, ".\n") ;
    exit (1) ;
  }
}

void get_params (file, argc, argv)
char **file ;
int argc ;
char *argv [] ;
{
  if (argc != 2)
  {
    fprintf (stderr, usage) ;
    exit (1) ;
  }
  *file = argv [1] ;
}

void main (argc, argv)
int argc ;
char *argv [] ;
{
  char *file ;
  FILE *stream ;
  int ptr, residue ;
  char buf [BUFFER_SIZE] ;
  int result ;


  get_params (&file, argc, argv) ;
  stream = fopen (file, "w") ;
  sys_assert (stream != NULL, "Failed to create file") ;
  
  ptr     = 0 ;
  residue = BUFFER_SIZE ;
  while (residue > 0)
  {
    result   = read (0, buf + ptr, residue) ;
    if (result == 0)
    {
      if (ptr == 0)
      {
	unlink (file) ;
	sys_assert (result == 0, "Unlink failed") ;
      }
      else
      {
	result = fwrite (buf, 1, ptr, stream) ;
	sys_assert (result == ptr, "Write failed") ;
      }
      exit (2) ;
    }
    sys_assert (result > 0, "Read failed") ;
    ptr     += result ;
    residue -= result ;
    printf ("result=%d, ptr=%d\n", result, ptr) ;
  }

  result = fwrite (buf, 1, ptr, stream) ;
  printf ("result=%d, ptr=%d\n", result, ptr) ;
  sys_assert (result == ptr, "Write failed") ;
  exit (0) ;
}

